HEADER_PEPTIDE = 'Peptide sequence'
HEADER_NO_PSM = '# PSMs'
HEADER_AREA = 'MS1 area (highest of all PSMs)'
HEADER_QVAL = 'q-value'
HEADER_QVAL_MODELED = 'q-value (linear modeled)'
HEADER_PEP = 'PEP'
HEADER_LINKED_PSMS = 'Files/scans for peptide'
HEADER_PROTEINS = 'Protein(s)'
HEADER_MASTERPROTEINS = 'Master protein(s)'
HEADER_NO_CONTENTPROTEINS = '# proteins in group'
HEADER_COVERAGES = 'Coverage(s)'
HEADER_GENES = 'Gene(s)'
HEADER_ASSOCIATED = 'Associated gene ID(s)'
HEADER_DESCRIPTIONS = 'Description(s)'
HEADER_SVM_SCORE = 'percolator svm-score'
