from app.readers.mzidplus import (PERCO_ORDER, P_PEPTIDEQ, P_PEPTIDEPEP)

DECOY_PREFIX = 'decoy_'

HEADER_SPECFILE = '#SpecFile'
HEADER_SCANNR = 'ScanNum'
HEADER_CHARGE = 'Charge'
HEADER_PEPTIDE = 'Peptide'
HEADER_PROTEIN = 'Protein'
HEADER_GENE = 'Gene'
HEADER_SYMBOL = 'Gene Symbol'
HEADER_DESCRIPTION = 'Description'
HEADER_PRECURSOR_MZ = 'Precursor'
HEADER_PRECURSOR_QUANT = 'MS1 area'
HEADER_MASTER_PROT = 'Master protein(s)'
HEADER_PG_CONTENT = 'Protein group(s) content'
HEADER_PG_AMOUNT_PROTEIN_HITS = 'Amount of matching proteins in group(s)'
HEADER_PG = [HEADER_MASTER_PROT, HEADER_PG_CONTENT,
             HEADER_PG_AMOUNT_PROTEIN_HITS]
HEADER_PEPTIDE_Q = P_PEPTIDEQ[1]
HEADER_PEPTIDE_PEP = P_PEPTIDEPEP[1]
HEADER_MSGFSCORE = 'MSGFScore'
HEADER_EVALUE = 'EValue'
HEADER_SETNAME = 'Biological set'
HEADER_RETENTION_TIME = 'Retention time(min)'
HEADER_INJECTION_TIME = 'Ion injection time(ms)'
SPECDATA_HEADER = [HEADER_SETNAME, HEADER_RETENTION_TIME, HEADER_INJECTION_TIME]
PERCO_HEADER = [x[1] for x in PERCO_ORDER]
